@@ -1,9 +1,9 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright(c) 2006 Brian Tarricone, <bjt23@cornell.edu>
- *  Copyright(c) 2006 Benedikt Meurer, <benny@xfce.org>
- *  Copyright(c) 2010 Jannis Pohlmann, <jannis@xfce.org>
+ *  Copyright(c) 2006      Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright(c) 2006      Benedikt Meurer, <benny@xfce.org>
+ *  Copyright(c) 2010-2011 Jannis Pohlmann, <jannis@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -44,8 +44,6 @@
 
 #include <libxfce4ui/libxfce4ui.h>
 
-#include <dbus/dbus-glib.h>
-
 #ifdef HAVE_THUNARX
 #include <thunarx/thunarx.h>
 #endif
@@ -53,7 +51,6 @@
 #include "xfdesktop-common.h"
 #include "xfdesktop-file-utils.h"
 #include "xfdesktop-special-file-icon.h"
-#include "xfdesktop-trash-proxy.h"
 
 struct _XfdesktopSpecialFileIconPrivate
 {
@@ -61,14 +58,14 @@ struct _XfdesktopSpecialFileIconPrivate
     GdkPixbuf *pix;
     gchar *tooltip;
     gint cur_pix_size;
+    GFileMonitor *monitor;
     GFileInfo *file_info;
     GFileInfo *filesystem_info;
     GFile *file;
     GdkScreen *gscreen;
     
     /* only needed for trash */
-    DBusGProxy *dbus_proxy;
-    gboolean trash_full;
+    gboolean trash_item_count;
 };
 
 static void xfdesktop_special_file_icon_finalize(GObject *obj);
@@ -88,6 +85,11 @@ static gboolean xfdesktop_special_file_icon_populate_context_menu(XfdesktopIcon
 static GFileInfo *xfdesktop_special_file_icon_peek_file_info(XfdesktopFileIcon *icon);
 static GFileInfo *xfdesktop_special_file_icon_peek_filesystem_info(XfdesktopFileIcon *icon);
 static GFile *xfdesktop_special_file_icon_peek_file(XfdesktopFileIcon *icon);
+static void xfdesktop_special_file_icon_changed(GFileMonitor *monitor,
+                                                GFile *file,
+                                                GFile *other_file,
+                                                GFileMonitorEvent event,
+                                                XfdesktopSpecialFileIcon *special_file_icon);
 
 #ifdef HAVE_THUNARX
 static void xfdesktop_special_file_icon_tfi_init(ThunarxFileInfoIface *iface);
@@ -153,10 +155,17 @@ xfdesktop_special_file_icon_finalize(GObject *obj)
     g_signal_handlers_disconnect_by_func(G_OBJECT(itheme),
                                          G_CALLBACK(xfdesktop_special_file_icon_invalidate_pixbuf),
                                          icon);
-    
+
     if(icon->priv->pix)
         g_object_unref(G_OBJECT(icon->priv->pix));
 
+    if(icon->priv->monitor) {
+        g_signal_handlers_disconnect_by_func(icon->priv->monitor,
+                                             G_CALLBACK(xfdesktop_special_file_icon_changed),
+                                             icon);
+        g_object_unref(icon->priv->monitor);
+    }
+    
     g_object_unref(icon->priv->file);
 
     if(icon->priv->file_info)
@@ -389,18 +398,15 @@ xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon)
             return NULL;
 
         if(XFDESKTOP_SPECIAL_FILE_ICON_TRASH == special_file_icon->priv->type) {
-            guint item_count = g_file_info_get_attribute_uint32(info, 
-                                                                G_FILE_ATTRIBUTE_TRASH_ITEM_COUNT);
-
-            if(item_count == 0) {
+            if(special_file_icon->priv->trash_item_count == 0) {
                 special_file_icon->priv->tooltip = g_strdup(_("Trash is empty"));
             } else {
                 special_file_icon->priv->tooltip = g_strdup_printf(g_dngettext(GETTEXT_PACKAGE,
                                                                                _("Trash contains one item"),
                                                                                _("Trash contains %d items"),
-                                                                               item_count), 
+                                                                               special_file_icon->priv->trash_item_count), 
 
-                                                                   item_count);
+                                                                   special_file_icon->priv->trash_item_count);
             }
         } else {
             const gchar *description;
@@ -436,94 +442,19 @@ xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon)
     return special_file_icon->priv->tooltip;
 }
 
-static void
-xfdesktop_special_file_icon_trash_handle_error(XfdesktopSpecialFileIcon *icon,
-                                               const gchar *method,
-                                               const gchar *message)
-{
-    GtkWidget *icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(icon));
-    GtkWidget *toplevel = gtk_widget_get_toplevel(icon_view);
-    GtkWidget *dlg = xfce_message_dialog_new(GTK_WINDOW(toplevel),
-                                             _("Trash Error"),
-                                             GTK_STOCK_DIALOG_WARNING,
-                                             _("Unable to contact the Xfce Trash service."),
-                                             _("Make sure you have a file manager installed that supports the Xfce Trash service, such as Thunar."),
-                                             GTK_STOCK_CLOSE,
-                                             GTK_RESPONSE_ACCEPT, NULL);
-    gtk_dialog_run(GTK_DIALOG(dlg));
-    gtk_widget_destroy(dlg);
-    
-    g_warning("org.xfce.Trash.%s failed: %s", method ? method : "??",
-              message ? message : "??");
-}
-
-static void
-xfdesktop_special_file_icon_trash_open_cb(DBusGProxy *proxy,
-                                          GError *error,
-                                          gpointer user_data)
-{
-    GtkWidget *icon_view, *toplevel;
-    
-    icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(user_data));
-    toplevel = gtk_widget_get_toplevel(icon_view);
-    xfdesktop_file_utils_set_window_cursor(GTK_WINDOW(toplevel),
-                                           GDK_LEFT_PTR);
-    
-    if(error) {
-        xfdesktop_special_file_icon_trash_handle_error(XFDESKTOP_SPECIAL_FILE_ICON(user_data),
-                                                       "DisplayTrash",
-                                                        error->message);
-    }
-    
-    g_object_unref(G_OBJECT(user_data));
-}
-
-static void
-xfdesktop_special_file_icon_trash_empty_cb(DBusGProxy *proxy,
-                                           GError *error,
-                                           gpointer user_data)
-{
-    if(error) {
-        xfdesktop_special_file_icon_trash_handle_error(XFDESKTOP_SPECIAL_FILE_ICON(user_data),
-                                                       "EmptyTrash",
-                                                        error->message);
-    }
-    
-    g_object_unref(G_OBJECT(user_data));
-}
-
 static void
 xfdesktop_special_file_icon_trash_open(GtkWidget *w,
                                        gpointer user_data)
 {
     XfdesktopSpecialFileIcon *file_icon = XFDESKTOP_SPECIAL_FILE_ICON(user_data);
+    GtkWidget *icon_view, *toplevel;
+
+    icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(file_icon));
+    toplevel = gtk_widget_get_toplevel(icon_view);
     
-    if(G_LIKELY(file_icon->priv->dbus_proxy)) {
-        gchar *display_name = gdk_screen_make_display_name(file_icon->priv->gscreen);
-        gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
-        
-        if(!xfdesktop_trash_proxy_display_trash_async(file_icon->priv->dbus_proxy,
-                                                      display_name, startup_id,
-                                                      xfdesktop_special_file_icon_trash_open_cb,
-                                                      file_icon))
-        {
-            xfdesktop_special_file_icon_trash_handle_error(file_icon,
-                                                           "DisplayTrash",
-                                                            NULL);
-        } else {
-            GtkWidget *icon_view, *toplevel;
-            
-            g_object_ref(G_OBJECT(file_icon));
-            
-            icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(file_icon));
-            toplevel = gtk_widget_get_toplevel(icon_view);
-            xfdesktop_file_utils_set_window_cursor(GTK_WINDOW(toplevel),
-                                                   GDK_WATCH);
-        }
-            
-        g_free(startup_id);
-        g_free(display_name);
-    }
+    xfdesktop_file_utils_open_folder(file_icon->priv->file,
+                                     file_icon->priv->gscreen,
+                                     GTK_WINDOW(toplevel));
 }
 
 static void
@@ -531,25 +462,13 @@ xfdesktop_special_file_icon_trash_empty(GtkWidget *w,
                                         gpointer user_data)
 {
     XfdesktopSpecialFileIcon *file_icon = XFDESKTOP_SPECIAL_FILE_ICON(user_data);
-    
-    if(G_LIKELY(file_icon->priv->dbus_proxy)) {
-        gchar *display_name = gdk_screen_make_display_name(file_icon->priv->gscreen);
-        gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
-        
-        if(!xfdesktop_trash_proxy_empty_trash_async(file_icon->priv->dbus_proxy,
-                                                    display_name, startup_id,
-                                                    xfdesktop_special_file_icon_trash_empty_cb,
-                                                    file_icon))
-        {
-            xfdesktop_special_file_icon_trash_handle_error(file_icon,
-                                                           "EmptyTrash",
-                                                            NULL);
-        } else
-            g_object_ref(G_OBJECT(file_icon));
+    GtkWidget *icon_view, *toplevel;
 
-        g_free(startup_id);
-        g_free(display_name);
-    }
+    icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(file_icon));
+    toplevel = gtk_widget_get_toplevel(icon_view);
+
+    xfdesktop_file_utils_empty_trash(file_icon->priv->gscreen,
+                                     GTK_WINDOW(toplevel));
 }
 
 static gboolean
@@ -590,7 +509,7 @@ xfdesktop_special_file_icon_populate_context_menu(XfdesktopIcon *icon,
         gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-    if(special_file_icon->priv->trash_full) {
+    if(special_file_icon->priv->trash_item_count > 0) {
         g_signal_connect(G_OBJECT(mi), "activate",
                          G_CALLBACK(xfdesktop_special_file_icon_trash_empty),
                          icon);
@@ -621,6 +540,57 @@ xfdesktop_special_file_icon_peek_file(XfdesktopFileIcon *icon)
     return XFDESKTOP_SPECIAL_FILE_ICON(icon)->priv->file;
 }
 
+static void
+xfdesktop_special_file_icon_changed(GFileMonitor *monitor,
+                                    GFile *file,
+                                    GFile *other_file,
+                                    GFileMonitorEvent event,
+                                    XfdesktopSpecialFileIcon *special_file_icon)
+{
+    g_return_if_fail(G_IS_FILE_MONITOR(monitor));
+    g_return_if_fail(G_IS_FILE(file));
+    g_return_if_fail(XFDESKTOP_IS_SPECIAL_FILE_ICON(special_file_icon));
+
+    /* release the old file information */
+    if(special_file_icon->priv->file_info) {
+        g_object_unref(special_file_icon->priv->file_info);
+        special_file_icon->priv->file_info = NULL;
+    }
+
+    /* release the old file system information */
+    if(special_file_icon->priv->filesystem_info) {
+        g_object_unref(special_file_icon->priv->filesystem_info);
+        special_file_icon->priv->filesystem_info = NULL;
+    }
+
+    /* reload the file information */
+    special_file_icon->priv->file_info = g_file_query_info(special_file_icon->priv->file, 
+                                                           XFDESKTOP_FILE_INFO_NAMESPACE,
+                                                           G_FILE_QUERY_INFO_NONE,
+                                                           NULL, NULL);
+
+    /* reload the file system information */
+    special_file_icon->priv->filesystem_info = g_file_query_filesystem_info(special_file_icon->priv->file,
+                                                                            XFDESKTOP_FILESYSTEM_INFO_NAMESPACE,
+                                                                            NULL, NULL);
+
+    /* update the trash full state */
+    if(special_file_icon->priv->file_info 
+       && special_file_icon->priv->type == XFDESKTOP_SPECIAL_FILE_ICON_TRASH) 
+    {
+        special_file_icon->priv->trash_item_count = g_file_info_get_attribute_uint32(special_file_icon->priv->file_info, 
+                                                                                     G_FILE_ATTRIBUTE_TRASH_ITEM_COUNT);
+    }
+
+    /* invalidate the tooltip */
+    g_free(special_file_icon->priv->tooltip);
+    special_file_icon->priv->tooltip = NULL;
+
+    /* update the icon */
+    xfdesktop_special_file_icon_invalidate_pixbuf(special_file_icon);
+    xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(special_file_icon));
+}
+
 /* public API */
 
 XfdesktopSpecialFileIcon *
@@ -667,10 +637,25 @@ xfdesktop_special_file_icon_new(XfdesktopSpecialFileIconType type,
                                                                             XFDESKTOP_FILESYSTEM_INFO_NAMESPACE,
                                                                             NULL, NULL);
 
+    if(type == XFDESKTOP_SPECIAL_FILE_ICON_TRASH) {
+        special_file_icon->priv->trash_item_count = g_file_info_get_attribute_uint32(special_file_icon->priv->file_info, 
+                                                                                     G_FILE_ATTRIBUTE_TRASH_ITEM_COUNT);
+    }
+
     g_signal_connect_swapped(G_OBJECT(gtk_icon_theme_get_for_screen(screen)),
                              "changed",
                              G_CALLBACK(xfdesktop_special_file_icon_invalidate_pixbuf),
                              special_file_icon);
+
+    special_file_icon->priv->monitor = g_file_monitor(special_file_icon->priv->file,
+                                                      G_FILE_MONITOR_NONE,
+                                                      NULL, NULL);
+    if(special_file_icon->priv->monitor) {
+        g_signal_connect(special_file_icon->priv->monitor,
+                         "changed",
+                         G_CALLBACK(xfdesktop_special_file_icon_changed),
+                         special_file_icon);
+    }
     
     return special_file_icon;
 }
